tankers = {}

tankers.active = {}
tankers.useMetric = false;
tankers.totalTankers = 0
tankers.checkInterval = 10
tankers.headingCorrection = 0

if env.mission.theatre == "Caucasus" then
	tankers.headingCorrection = 0.10472 -- radians (6 degrees)
elseif env.mission.theatre == "Nevada" then
	tankers.headingCorrection = 0 -- radians (0 degrees)
elseif env.mission.theatre == "Normandy" then
	tankers.headingCorrection = 0 -- radians (0 degrees)
end

----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- The following three functions were taken directly from MIST 4.3.74. I DID NOT write
-- these functions. I only changed the names of the functions from "mist" to "tankers".
-- Since these were the only MIST functions that this particular script required, I extracted
-- these just to simplify things for the mission builder. I believe this complies with
-- the MIST Usage Rights stated here:
-- https://forums.eagle.ru/showthread.php?t=98616

-- By the way...MIST is an amazing bit of code!

	function tankers.deepCopy(object)
		local lookup_table = {}
		local function _copy(object)
			if type(object) ~= "table" then
				return object
			elseif lookup_table[object] then
				return lookup_table[object]
			end
			local new_table = {}
			lookup_table[object] = new_table
			for index, value in pairs(object) do
				new_table[_copy(index)] = _copy(value)
			end
			return setmetatable(new_table, getmetatable(object))
		end
		return _copy(object)
	end

	function tankers.getNorthCorrection(gPoint)	--gets the correction needed for true north
		local point = tankers.deepCopy(gPoint)
		if not point.z then --Vec2; convert to Vec3
			point.z = point.y
			point.y = 0
		end
		local lat, lon = coord.LOtoLL(point)
		local north_posit = coord.LLtoLO(lat + 1, lon)
		return math.atan2(north_posit.z - point.z, north_posit.x - point.x)
	end

	function tankers.getHeading(unit, rawHeading)
		local unitpos = unit:getPosition()
		if unitpos then
			local Heading = math.atan2(unitpos.x.z, unitpos.x.x)
			if not rawHeading then
				Heading = Heading + tankers.getNorthCorrection(unitpos.p)
			end
			if Heading < 0 then
				Heading = Heading + 2*math.pi	-- put heading in range of 0 to 2*pi
			end
			return Heading
		end
	end

----------------------------------------------------------------------------------
----------------------------------------------------------------------------------

function tankers.getTacanFreq(channel)

	local frequency = 0
	
	if channel < 64 then
		frequency = channel + 1087
	else
		frequency = channel + 961
	end

	return frequency * 1000000
end

function tankers.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacan)

	local validData = true

	if Unit.getByName(referenceObjectName) == nil then
		if StaticObject.getByName(referenceObjectName) == nil then
			trigger.action.outText("Tanker reference object does not exist:  " .. referenceObjectName  , 15)
			validData = false
		end
	end

	if aircraftType ~= "S-3B Tanker" and aircraftType ~= "KC-135" and aircraftType ~= "KC130" and aircraftType ~= "IL-78M" then
		trigger.action.outText("Invalid tanker aircraft type:  " .. aircraftType  , 15)
		trigger.action.outText("- Must be S-3B Tanker, KC-135, KC-130, or IL-78M", 15)
		validData = false
	end

	local countryNumber = country.id[string.upper(countryID)]
	
	if coalition.getCountryCoalition(countryNumber) == coalition.side.BLUE then
		if callsignName ~= "Texaco" and callsignName ~= "Arco" and callsignName ~= "Shell" then
			trigger.action.outText("Invalid tanker callsign:  " .. callsignName  , 15)
			trigger.action.outText("- Must be Texaco, Arco, or Shell for Blue coalition", 15)
			validData = false
		end
	elseif coalition.getCountryCoalition(countryNumber) == coalition.side.RED then
		if tonumber(callsignName) == nil or tonumber(callsignName) < 1 or tonumber(callsignName) > 9 then
			trigger.action.outText("Invalid tanker callsign:  " .. callsignName  , 15)
			trigger.action.outText("- Must be a number between 0 and 9 for Red coalition", 15)
			validData = false
		end
	else
		trigger.action.outText("Invalid tanker country: " .. countryID)
		validData = false
	end

	if tacan ~= false and tacan ~= "false" then

		local tacanChannel = tonumber(string.sub(tacan, 1, string.len(tacan) - 1))
		local tacanChannelMode = string.upper(string.sub(tacan, -1))

		if tacanChannel == nil or tacanChannel < 1 or tacanChannel > 126 or (tacanChannelMode ~= "X" and tacanChannelMode ~= "Y") then
			trigger.action.outText("Invalid tanker TACAN channel:  " .. tacan  , 15)
			validData = false
		end
	end
	
	return validData

end

function tankers.addTanker(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn)

	if not tankers.useMetric then
		altitude = altitude * 0.3048 -- convert feet to meters
		trueAirspeed = trueAirspeed * 0.514444 -- conver knots to meters per second
		trackStart = trackStart * 1852 -- convert nm to meters
		trackLength = trackLength * 1852 -- convert nm to meters
	end

	tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn, nil, nil, nil)

end

function tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn, groupName, unitName, tableIndex)

	if aircraftType == "KC-130" then
		aircraftType = "KC130"
	elseif aircraftType == "S-3B" then
		aircraftType = "S-3B Tanker"
	elseif aircraftType == "IL-78" then
		aircraftType = "IL-78M"
	end
		
	if tankers.totalTankers < 99 and tankers.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacan) then
	
		local countryNumber = country.id[string.upper(countryID)]

		tankers.totalTankers = tankers.totalTankers + 1
		
		local referenceObject = Unit.getByName(referenceObjectName)
		if referenceObject == nil then
			referenceObject = StaticObject.getByName(referenceObjectName)
		end
		
		local referencePoint = referenceObject:getPoint()
		local referenceX = referencePoint.x
		local referenceY = referencePoint.z
		
		local referenceHeading = (tankers.getHeading(referenceObject) + tankers.headingCorrection) % (2 * 3.14159265359)
		
		local referenceHeadingCos = math.cos(referenceHeading)
		local referenceHeadingSin = math.sin(referenceHeading)
		
		local spawnX = (referenceHeadingCos * (trackStart - 926)) + referenceX -- 926 meters = 0.5 nm
		local spawnY = (referenceHeadingSin * (trackStart - 926)) + referenceY -- 926 meters = 0.5 nm

		local trackStartX = (referenceHeadingCos * trackStart) + referenceX
		local trackStartY = (referenceHeadingSin * trackStart) + referenceY

		local trackEndX = (referenceHeadingCos * (trackStart + trackLength)) + referenceX
		local trackEndY = (referenceHeadingSin * (trackStart + trackLength)) + referenceY

		local fuel = 0
		local flares = 0
		local chaff = 0
		local totalTankers = 0
		
		local onboardNumber = string.format("%03d",tankers.totalTankers)

		if aircraftType == "S-3B Tanker" then
			--fuel = "5500"
			fuel =5500
			flares = 30
			chaff = 30
		elseif aircraftType == "KC-135" then
			fuel = 90700
			flares = 0
			chaff = 0
		elseif aircraftType == "KC130" then
			fuel = 20830
			flares = 75
			chaff = 90
		elseif aircraftType == "IL-78M" then
			--fuel = "90000
			fuel = 90000
			flares = 96
			chaff = 96
		end
		
		-- local callsign1 = (tankers.totalTankers % 3) + 1

		local callsign = nil
		local callsignText = ""

		local callsign1 = 0
		local callsign2 = math.floor(tankers.totalTankers / 10)
		local callsign3 = tankers.totalTankers % 10

		
		if coalition.getCountryCoalition(countryNumber) == coalition.side.BLUE then
			if callsignName == "Texaco" then
				callsign1 = 1
			elseif callsignName == "Arco" then
				callsign1 = 2
			elseif callsignName == "Shell" then
				callsign1 = 3
			else
				callsign1 = tonumber(callsignName)
			end
			callsign =
			{
				[1] = callsign1,
				[2] = callsign2,
				[3] = callsign3,
				["name"] = callsignName .. tostring(callsign2) .. tostring(callsign3),
			}
			onboardNumber = tankers.totalTankers + (callsign1 * 100)
			callsignText = callsignName .. " " .. callsign2 .. callsign3
			else 
			callsign = tonumber(callsignName .. callsign2 .. callsign3)
			onboardNumber = callsign
			callsignText = callsign
		end
		
		local tankerGroupName = groupName
		local tankerUnitName = unitName

		if tankerGroupName == nil then
			tankerGroupName = "Tanker_Group_" .. tankers.totalTankers
		end

		if tankerUnitName == nil then
			tankerUnitName = "Tanker_Unit_" .. tankers.totalTankers
		end

		local tacanChannel = 0
		local tacanChannelMode = ""
		local tacanCallsign = ""
		local tacanFrequency = 0
		local tacanSystem = 0

		if tacan ~= false and tacan ~= "false" then
			tacanChannel = tonumber(string.sub(tacan, 1, string.len(tacan) - 1))
			tacanChannelMode = string.upper(string.sub(tacan, -1))
			tacanCallsign = "TK" .. tacanChannel .. tacanChannelMode
			tacanFrequency = tankers.getTacanFreq(tacanChannel)
			if tacanChannelMode == "X" then
				tacanSystem = 4
			elseif tacanChannelMode == "Y" then
				tacanSystem = 5
			end

		end

		local groupData = 
		{
			["modulation"] = 0,
			["tasks"] = 
			{
			}, -- end of ["tasks"]
			["radioSet"] = false,
			["task"] = "Refueling",
			["uncontrolled"] = false,
			["route"] = 
			{
				["points"] = 
				{
					[1] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = true,
										["id"] = "Tanker",
										["enabled"] = true,
										["params"] = 
										{
										}, -- end of ["params"]
									}, -- end of [1]
									[2] = 
									{
										["enabled"] = true,
										["auto"] = true,
										["id"] = "WrappedAction",
										["number"] = 2,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "ActivateBeacon",
												["params"] = 
												{
													["type"] = 4,
													["AA"] = true,
													["callsign"] = tacanCallsign,
													["system"] = tacanSystem,
													["channel"] = tacanChannel,
													["modeChannel"] = tacanChannelMode,
													["bearing"] = true,
													["frequency"] = tacanFrequency
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [2]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = spawnY,
						["x"] = spawnX,
						["name"] = "Tanker_Wpt_1_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [1]
					[2] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = false,
										["id"] = "Orbit",
										["enabled"] = true,
										["params"] = 
										{
											["altitude"] = altitude,
											["pattern"] = "Race-Track",
											["speed"] = trueAirspeed,
											["speedEdited"] = true,
										}, -- end of ["params"]
									}, -- end of [1]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = trackStartY,
						["x"] = trackStartX,
						["name"] = "Tanker_Wpt_2_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [2]
					[3] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = false,
										["id"] = "WrappedAction",
										["enabled"] = true,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "Option",
												["params"] = 
												{
													["value"] = true,
													["name"] = 6,
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [1]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = trackEndY,
						["x"] = trackEndX,
						["name"] = "Tanker_Wpt_3_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [3]
					[4] = 
					{
						["alt"] = altitude / 2,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = referenceY,
						["x"] = referenceX,
						["name"] = "Tanker_Wpt_4_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [4]
				}, -- end of ["points"]
			}, -- end of ["route"]
			--["groupId"] = 9999,
			["hidden"] = false,
			["units"] = 
			{
				[1] = 
				{
					["alt"] = altitude,
					["alt_type"] = "BARO",
					--["livery_id"] = "usaf standard",
					["skill"] = "Excellent",
					["speed"] = trueAirspeed,
					["type"] = aircraftType,
					["psi"] = -referenceHeading,
					["y"] = spawnY,
					["x"] = spawnX,
					["name"] = tankerUnitName,
					["payload"] = 
					{
						["pylons"] = 
						{
						}, -- end of ["pylons"]
						["fuel"] = fuel,
						["flare"] = flares,
						["chaff"] = chaff,
						["gun"] = 100,
					}, -- end of ["payload"]
					["heading"] = referenceHeading,
					["callsign"] = callsign,
					["onboard_num"] = onboardNumber,
				}, -- end of [1]
			}, -- end of ["units"]
			["y"] = spawnY,
			["x"] = spawnX,
			["name"] = tankerGroupName,
			["communication"] = true,
			["start_time"] = 0,
			["frequency"] = radioFrequency,
		} -- end of groupData

		coalition.addGroup(countryNumber, Group.Category.AIRPLANE, groupData)

		local onStationAltitude = altitude
		if not tankers.useMetric then
			onStationAltitude = math.floor(onStationAltitude * 3.28084) -- convert meters to feet
		end
		trigger.action.outTextForCoalition(coalition.getCountryCoalition(countryNumber), "Tanker " .. callsignText .. " (" .. aircraftType .. ") on station at " .. onStationAltitude, 10)
		if tableIndex == nil then
			table.insert(tankers.active, {aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn, groupData.name, groupData.units[1].name})
		else
			table.insert(tankers.active, tableIndex, {aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn, groupData.name, groupData.units[1].name})
		end
	end

end
	
function tankers.isDead(groupName)

	if (Group.getByName(groupName) and Group.getByName(groupName):isExist() == false)
			or (Group.getByName(groupName) and #Group.getByName(groupName):getUnits() < 1)
			or not Group.getByName(groupName) then
		return true
	else
		return false
	end

end

function tankers.checkRespawn(arg, time)

	for i=1, #tankers.active do
		local aircraftType = tankers.active[i][1]
		local referenceObjectName = tankers.active[i][2]
		local countryID = tankers.active[i][3]
		local altitude = tankers.active[i][4]
		local trueAirspeed = tankers.active[i][5]
		local trackStart = tankers.active[i][6]
		local trackLength = tankers.active[i][7]
		local radioFrequency = tankers.active[i][8]
		local callsignName = tankers.active[i][9]
		local tacan = tankers.active[i][10]
		local respawn = tankers.active[i][11]
		local groupName = tankers.active[i][12]
		local unitName = tankers.active[i][13]
	
		if tankers.isDead(groupName) or Unit.getByName(unitName):getPosition().p.y < altitude - 600 then  -- 600 meters = 2000 feet
			if respawn then
				table.remove(tankers.active, i)
				tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacan, respawn, groupName, unitName, i)				
			else
				table.remove(tankers.active, i)
			end
		end
	end

	return time + tankers.checkInterval

end

do

	--tankers.addTanker("IL-78M", "Tire1", "Russia", 20000, 300, 20, 30, 270, 5, "10X", true)
	--tankers.addTanker("KC-130", "CVN-70", "USA", 15000, 300, 1, 20, 251, "Texaco", "12X", true)

	timer.scheduleFunction(tankers.checkRespawn, nil, timer.getTime() + tankers.checkInterval)

end